@@ -1,34 +1,37 @@
 #!/usr/bin/python3
 import gettext
-import gi
 import locale
 import os
-import re
-import setproctitle
 import shutil
-import subprocess
-import warnings
 import sys
 import time
 import traceback
+import warnings
+from functools import partial
+from pathlib import Path
+
+# Force X11 on a Wayland session
+if "WAYLAND_DISPLAY" in os.environ:
+    os.environ["WAYLAND_DISPLAY"] = ""
 
 # Suppress GTK deprecation warnings
 warnings.filterwarnings("ignore")
 
+import gi
 gi.require_version("Gtk", "3.0")
 gi.require_version('XApp', '1.0')
 from gi.repository import Gtk, Gdk, Gio, XApp, GdkPixbuf, GLib, Pango
 
-from common import *
-
 import mpv
-
+import requests
+import setproctitle
 from imdb import IMDb
-
-from functools import partial
-
 from unidecode import unidecode
 
+from common import Manager, Provider, BADGES, MOVIES_GROUP, PROVIDERS_PATH, SERIES_GROUP, TV_GROUP,\
+    async_function, idle_function
+
+
 setproctitle.setproctitle("hypnotix")
 
 # i18n
@@ -99,6 +102,7 @@ class MainWindow():
         self.active_channel = None
         self.fullscreen = False
         self.latest_search_bar_text = None
+        self.visible_search_results = 0
         self.mpv = None
         self.ia = IMDb()
 
@@ -389,6 +393,7 @@ class MainWindow():
                 self.channels_flowbox.remove(child)
             for channel in channels:
                 button = Gtk.Button()
+                button.set_tooltip_text(channel.name)
                 button.connect("clicked", self.on_channel_button_clicked, channel)
                 label = Gtk.Label()
                 label.set_text(channel.name)
@@ -403,6 +408,7 @@ class MainWindow():
                 button.add(box)
                 self.channels_flowbox.add(button)
             self.channels_flowbox.show_all()
+            self.visible_search_results = len(self.channels_flowbox.get_children())
             if len(logos_to_refresh) > 0:
                 self.download_channel_logos(logos_to_refresh)
         else:
@@ -415,6 +421,7 @@ class MainWindow():
             self.vod_flowbox.remove(child)
         for item in items:
             button = Gtk.Button()
+            button.set_tooltip_text(item.name)
             if self.content_type == MOVIES_GROUP:
                 button.connect("clicked", self.on_vod_movie_button_clicked, item)
             else:
@@ -465,6 +472,7 @@ class MainWindow():
             for episode_name in season.episodes.keys():
                 episode = season.episodes[episode_name]
                 button = Gtk.Button()
+                button.set_tooltip_text(episode_name)
                 button.connect("clicked", self.on_episode_button_clicked, episode)
                 label = Gtk.Label()
                 label.set_text(_("Episode %s") % episode_name)
@@ -572,15 +580,20 @@ class MainWindow():
             search_bar_text = unidecode(self.search_bar.get_text()).lower()
             label_text = unidecode(child.get_children()[0].get_children()[0].get_children()[1].get_text()).lower()
             if search_bar_text in label_text:
+                self.visible_search_results += 1
                 return True
             else:
                 return False
 
+        self.visible_search_results = 0
         self.channels_flowbox.set_filter_func(filter_func)
         if not self.channels_flowbox.get_children():
             self.show_channels(self.active_provider.channels)
-        self.status(None)
         print("Filtering %d channel names containing the string '%s'..." % (len(self.channels_flowbox.get_children()), self.latest_search_bar_text))
+        if self.visible_search_results == 0:
+            self.status(_("No channels found"))
+        else:
+            self.status(None)
         self.search_bar.set_sensitive(True)
         self.search_bar.grab_focus_without_selecting()
         self.navigate_to("channels_page")
@@ -591,6 +604,7 @@ class MainWindow():
         for child in self.channels_flowbox.get_children():
             self.channels_flowbox.remove(child)
         self.channels_flowbox.invalidate_filter()
+        self.visible_search_results = 0
 
     @idle_function
     def navigate_to(self, page, name=""):
@@ -604,18 +618,18 @@ class MainWindow():
             self.headerbar.set_title("Hypnotix")
             if provider == None:
                 self.headerbar.set_subtitle(_("No provider selected"))
-                self.tv_label.set_text("TV Channels (%d)" % 0)
-                self.movies_label.set_text("Movies (%d)" % 0)
-                self.series_label.set_text("Series (%d)" % 0)
+                self.tv_label.set_text(_("TV Channels (%d)") % 0)
+                self.movies_label.set_text(_("Movies (%d)") % 0)
+                self.series_label.set_text(_("Series (%d)") % 0)
                 self.preferences_button.set_sensitive(False)
                 self.tv_button.set_sensitive(False)
                 self.movies_button.set_sensitive(False)
                 self.series_button.set_sensitive(False)
             else:
                 self.headerbar.set_subtitle(provider.name)
-                self.tv_label.set_text("TV Channels (%d)" % len(provider.channels))
-                self.movies_label.set_text("Movies (%d)" % len(provider.movies))
-                self.series_label.set_text("Series (%d)" % len(provider.series))
+                self.tv_label.set_text(_("TV Channels (%d)") % len(provider.channels))
+                self.movies_label.set_text(_("Movies (%d)") % len(provider.movies))
+                self.series_label.set_text(_("Series (%d)") % len(provider.series))
                 self.preferences_button.set_sensitive(True)
                 self.tv_button.set_sensitive(len(provider.channels) > 0)
                 self.movies_button.set_sensitive(len(provider.movies) > 0)
@@ -950,11 +964,21 @@ class MainWindow():
             button.set_relief(Gtk.ReliefStyle.NONE)
             button.connect("clicked", self.on_edit_button_clicked, provider)
             image = Gtk.Image()
-            image.set_from_icon_name("list-edit-symbolic", Gtk.IconSize.BUTTON)
+            image.set_from_icon_name("xapp-edit-symbolic", Gtk.IconSize.BUTTON)
             button.set_tooltip_text(_("Edit"))
             button.add(image)
             box.pack_start(button, False, False, 0)
 
+            # Clear icon cache button
+            button = Gtk.Button()
+            button.set_relief(Gtk.ReliefStyle.NONE)
+            button.connect("clicked", self.on_clear_icon_cache_button_clicked, provider)
+            image = Gtk.Image()
+            image.set_from_icon_name("edit-clear-symbolic", Gtk.IconSize.BUTTON)
+            button.set_tooltip_text(_("Clear icon cache"))
+            button.add(image)
+            box.pack_start(button, False, False, 0)
+
             # Remove button
             button = Gtk.Button()
             button.set_relief(Gtk.ReliefStyle.NONE)
@@ -1002,6 +1026,12 @@ class MainWindow():
         self.info_window.hide()
         return True
 
+    @async_function
+    def on_clear_icon_cache_button_clicked(self, widget, provider: Provider):
+        for channel in provider.channels:
+            if channel.logo_path:
+                Path(channel.logo_path).unlink(missing_ok=True)
+
     def on_delete_button_clicked(self, widget, provider):
         self.navigate_to("delete_page", provider.name)
         self.marked_provider = provider
@@ -1283,7 +1313,7 @@ class MainWindow():
 
     @async_function
     def reload(self, page=None, refresh=False):
-        self.status("Loading providers...")
+        self.status(_("Loading providers..."))
         self.providers = []
         for provider_info in self.settings.get_strv("providers"):
             try:
@@ -1296,14 +1326,14 @@ class MainWindow():
                 if provider.type_id != "xtream":
                     # Download M3U
                     if refresh:
-                        self.status("Downloading playlist...", provider)
+                        self.status(_("Downloading playlist..."), provider)
                     else:
-                        self.status("Getting playlist...", provider)
+                        self.status(_("Getting playlist..."), provider)
                     ret = self.manager.get_playlist(provider, refresh=refresh)
                     if ret:
-                        self.status("Checking playlist...", provider)
+                        self.status(_("Checking playlist..."), provider)
                         if (self.manager.check_playlist(provider)):
-                            self.status("Loading channels...", provider)
+                            self.status(_("Loading channels..."), provider)
                             self.manager.load_channels(provider)
                             if provider.name == self.settings.get_string("active-provider"):
                                 self.active_provider = provider
@@ -1311,16 +1341,29 @@ class MainWindow():
                             print("%s: %d channels, %d groups, %d series, %d movies" % (provider.name, \
                                 len(provider.channels), len(provider.groups), len(provider.series), len(provider.movies)))
                     else:
-                        self.status("Failed to Download playlist from {}".format(provider.name),provider)
+                        self.status(_("Failed to Download playlist from {}").format(provider.name), provider)
 
                 else:
                     # Load xtream class
                     from xtream import XTream
                     # Download via Xtream
-                    self.x = XTream(provider.name,provider.username,provider.password,provider.url,os.path.expanduser("~/.hypnotix/providers"))
+                    self.x = XTream(provider.name,
+                                    provider.username,
+                                    provider.password,
+                                    provider.url,
+                                    hide_adult_content=False,
+                                    cache_path=PROVIDERS_PATH
+                                    )
                     if self.x.auth_data != {}:
                         print("XTREAM `{}` Loading Channels".format(provider.name))
+                        # Save default cursor
+                        current_cursor = self.window.get_window().get_cursor()
+                        # Set waiting cursor
+                        self.window.get_window().set_cursor(Gdk.Cursor.new_from_name(Gdk.Display.get_default(), 'wait'))
+                        # Load data
                         self.x.load_iptv()
+                        # Restore default cursor
+                        self.window.get_window().set_cursor(current_cursor)
                         # Inform Provider of data
                         provider.channels = self.x.channels
                         provider.movies = self.x.movies
@@ -1398,8 +1441,18 @@ class MainWindow():
         while not self.mpv_drawing_area.get_window() and not Gtk.events_pending():
             time.sleep(0.1)
 
-        self.mpv = mpv.MPV(**options, script_opts='osc-layout=box,osc-seekbarstyle=bar,osc-deadzonesize=0,osc-minmousemove=3', input_default_bindings=True, \
-             input_vo_keyboard=True,osc=True, ytdl=True, wid=str(self.mpv_drawing_area.get_window().get_xid()))
+        osc = True
+        if "osc" in options:
+            # To prevent 'multiple values for keyword argument'!
+            osc = options.pop("osc") != "no"
+
+        self.mpv = mpv.MPV(**options,
+                           script_opts="osc-layout=box,osc-seekbarstyle=bar,osc-deadzonesize=0,osc-minmousemove=3",
+                           input_default_bindings=True,
+                           input_vo_keyboard=True,
+                           osc=osc,
+                           ytdl=True,
+                           wid=str(self.mpv_drawing_area.get_window().get_xid()))
 
     def on_mpv_drawing_area_draw(self, widget, cr):
         cr.set_source_rgb(0.0, 0.0, 0.0)
