@@ -16,23 +16,19 @@ Github: superolmo
 
 """
 
-__version__ = '0.4.0'
+__version__ = '0.5.0'
 __author__ = 'Claudio Olmi'
 
-from typing import List
-import requests
+import json
+import re  # used for URL validation
 import time
 from os import path as osp
 from os import makedirs
-from os import remove
+from timeit import default_timer as timer  # Timing xtream json downloads
+from typing import List, Tuple
 
-# Timing xtream json downloads
-from timeit import default_timer as timer, timeit
+import requests
 
-import json
-
-# used for URL validation
-import re
 
 class Channel():
     # Required by Hypnotix
@@ -47,7 +43,7 @@ class Channel():
     # XTream
     stream_type = ""
     group_id = ""
-    is_adult = ""
+    is_adult = 0
     added = ""
     epg_channel_id = ""
     added = ""
@@ -75,13 +71,13 @@ class Channel():
             self.id = stream_info['stream_id']
             self.name = stream_name
             self.logo = stream_info['stream_icon']
-            self.logo_path = xtream.get_logo_local_path(self.logo)
+            self.logo_path = xtream._get_logo_local_path(self.logo)
             self.group_title = group_title
             self.title = stream_name
 
             # Check if category_id key is available
             if "category_id" in stream_info.keys():
-                self.group_id = stream_info['category_id']
+                self.group_id = int(stream_info['category_id'])
 
             if stream_type == "live":
                 stream_extension = "ts"
@@ -90,7 +86,7 @@ class Channel():
                 self.is_adult = 0
                 # Check if is_adult key is available
                 if "is_adult" in stream_info.keys():
-                    self.is_adult = stream_info['is_adult']
+                    self.is_adult = int(stream_info['is_adult'])
 
                 # Check if epg_channel_id key is available
                 if "epg_channel_id" in stream_info.keys():
@@ -112,7 +108,7 @@ class Channel():
                 )
 
             # Check that the constructed URL is valid
-            if not xtream.validate_url(self.url):
+            if not xtream._validate_url(self.url):
                 print("{} - Bad URL? `{}`".format(self.name, self.url))
 
     def export_json(self):
@@ -159,7 +155,7 @@ class Group():
 
         # Check if category_id key is available
         if "category_id" in group_info.keys():
-            self.group_id = group_info['category_id']
+            self.group_id = int(group_info['category_id'])
 
 class Episode():
     # Required by Hypnotix
@@ -184,7 +180,7 @@ class Episode():
         self.av_info = episode_info['info']
 
         self.logo = series_info['cover']
-        self.logo_path = xtream.get_logo_local_path(self.logo)
+        self.logo_path = xtream._get_logo_local_path(self.logo)
 
 
         self.url = "{}/series/{}/{}/{}.{}".format(
@@ -196,7 +192,7 @@ class Episode():
             )
 
         # Check that the constructed URL is valid
-        if not xtream.validate_url(self.url):
+        if not xtream._validate_url(self.url):
             print("{} - Bad URL? `{}`".format(self.name, self.url))
 
 class Serie():
@@ -221,14 +217,14 @@ class Serie():
         # Required by Hypnotix
         self.name = series_info['name']
         self.logo = series_info['cover']
-        self.logo_path = xtream.get_logo_local_path(self.logo)
+        self.logo_path = xtream._get_logo_local_path(self.logo)
 
         self.seasons = {}
         self.episodes = {}
 
         # Check if category_id key is available
         if "series_id" in series_info.keys():
-            self.series_id = series_info['series_id']
+            self.series_id = int(series_info['series_id'])
 
         # Check if plot key is available
         if "plot" in series_info.keys():
@@ -269,13 +265,17 @@ class XTream():
     series = []
     movies = []
 
+    state = {'authenticated': False, 'loaded': False}
+
+    hide_adult_content = False
+
     catch_all_group = Group(
         {
             "category_id": "9999",
             "category_name":"xEverythingElse",
             "parent_id":0
         },
-        live_type
+        ""
     )
     # If the cached JSON file is older than threshold_time_sec then load a new
     # JSON dictionary from the provider
@@ -286,6 +286,7 @@ class XTream():
                 provider_username: str,
                 provider_password: str,
                 provider_url: str,
+                hide_adult_content: bool = False,
                 cache_path: str = ""
                 ):
         """Initialize Xtream Class
@@ -295,6 +296,7 @@ class XTream():
             provider_username (str):            User name of the IPTV provider
             provider_password (str):            Password of the IPTV provider
             provider_url      (str):            URL of the IPTV provider
+            hide_adult_content(bool):           When `True` hide stream that are marked for adult
             cache_path        (str, optional):  Location where to save loaded files. Defaults to empty string.
 
         Returns: XTream Class Instance
@@ -308,11 +310,13 @@ class XTream():
         self.password = provider_password
         self.name = provider_name
         self.cache_path = cache_path
+        self.hide_adult_content = hide_adult_content
+
         # if the cache_path is specified, test that it is a directory
         if self.cache_path != "":
             # If the cache_path is not a directory, clear it
             if not osp.isdir(self.cache_path):
-                print("Cache Path is not a directory, using default '~/.xtream-cache/'")
+                print(" - Cache Path is not a directory, using default '~/.xtream-cache/'")
                 self.cache_path == ""
 
         # If the cache_path is still empty, use default
@@ -365,7 +369,7 @@ class XTream():
             return search_result
 
 
-    def slugify(self, string: str) -> str:
+    def _slugify(self, string: str) -> str:
         """Normalize string
 
         Normalizes string, converts to lowercase, removes non-alpha characters,
@@ -377,9 +381,9 @@ class XTream():
         Returns:
             str: Normalized String
         """
-        return "".join(x.lower() for x in string if x.isalnum())
+        return "".join(x.lower() for x in string if x.isprintable())
 
-    def validate_url(self, url: str) -> bool:
+    def _validate_url(self, url: str) -> bool:
         regex = re.compile(
             r'^(?:http|ftp)s?://' # http:// or https://
             r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|' #domain...
@@ -390,7 +394,7 @@ class XTream():
 
         return re.match(regex, url) is not None
 
-    def get_logo_local_path(self, logo_url: str) -> str:
+    def _get_logo_local_path(self, logo_url: str) -> str:
         """Convert the Logo URL to a local Logo Path
 
         Args:
@@ -401,51 +405,52 @@ class XTream():
         """
         local_logo_path = None
         if logo_url != None:
-            if not self.validate_url(logo_url):
+            if not self._validate_url(logo_url):
                 logo_url = None
             else:
                 local_logo_path = osp.join(self.cache_path, "{}-{}".format(
-                    self.slugify(self.name),
-                    self.slugify(osp.split(logo_url)[-1])
+                    self._slugify(self.name),
+                    self._slugify(osp.split(logo_url)[-1])
                     )
                 )
         return local_logo_path
 
-
-    # If you want to limit the displayed output data,
-    # you can use params[offset]=X & params[items_per_page]=X on your call.
-
-    # Authentication returns information about the account and server:
     def authenticate(self):
-        self.auth_data = {}
-        try:
-            r = requests.get(
-                self.get_authenticate_URL()
-                )
-            if r.ok:
-                self.auth_data = r.json()
-                self.authorization = {
-                    "username": self.auth_data["user_info"]["username"],
-                    "password": self.auth_data["user_info"]["password"]
-                }
-            else:
-                print("Provider `{}` could not be loaded. Reason: `{} {}`".format(self.name,r.status_code, r.reason))
-        except requests.exceptions.ConnectionError:
-            # If connection refused
-            print("{} - Connection refused URL: {}".format(self.name, self.server))
+        """Login to provider
+        """
+        # If we have not yet successfully authenticated, attempt authentication
+        if (self.state['authenticated'] == False):
+            # Erase any previous data
+            self.auth_data = {}
+            try:
+                # Request authentication, wait 4 seconds maximum
+                r = requests.get(self.get_authenticate_URL(), timeout=(4))
+                # If the answer is ok, process data and change state
+                if r.ok:
+                    self.auth_data = r.json()
+                    self.authorization = {
+                        "username": self.auth_data["user_info"]["username"],
+                        "password": self.auth_data["user_info"]["password"]
+                    }
+                    self.state['authenticated'] = True
+                else:
+                    print("Provider `{}` could not be loaded. Reason: `{} {}`".format(self.name,r.status_code, r.reason))
+            except requests.exceptions.ConnectionError:
+                # If connection refused
+                print("{} - Connection refused URL: {}".format(self.name, self.server))
 
-    def load_from_file(self, filename) -> dict:
-        """Try to load the distionary from file
+    def _load_from_file(self, filename) -> dict:
+        """Try to load the dictionary from file
 
         Args:
             filename ([type]): File name containing the data
 
         Returns:
-            dict: Dictionary is found and no errors, None is file does not exists
+            dict: Dictionary if found and no errors, None if file does not exists
         """
         #Build the full path
         full_filename = osp.join(self.cache_path, "{}-{}".format(
-                self.slugify(self.name),
+                self._slugify(self.name),
                 filename
         ))
 
@@ -466,14 +471,14 @@ class XTream():
                         if len(my_data) == 0:
                             my_data = None
                 except Exception as e:
-                    print("Could not save to file `{}`: e=`{}`".format(
+                    print(" - Could not load from file `{}`: e=`{}`".format(
                         full_filename, e
                     ))
             return my_data
         else:
             return None
 
-    def save_to_file(self, data_list: dict, filename: str) -> bool:
+    def _save_to_file(self, data_list: dict, filename: str) -> bool:
         """Save a dictionary to file
 
         This function will overwrite the file if already exists
@@ -485,23 +490,27 @@ class XTream():
         Returns:
             bool: True if successfull, False if error
         """
-        #Build the full path
-        full_filename = osp.join(self.cache_path, "{}-{}".format(
-                self.slugify(self.name),
-                filename
-        ))
-        # If the path makes sense, save the file
-        json_data = json.dumps(data_list, ensure_ascii=False)
-        try:
-            with open(full_filename, mode='wt', encoding='utf-8') as myfile:
-                myfile.write(json_data)
-        except Exception as e:
-            print("Could not save to file `{}`: e=`{}`".format(
-                full_filename, e
-            ))
-            return False
+        if data_list != None:
 
-        return True
+            #Build the full path
+            full_filename = osp.join(self.cache_path, "{}-{}".format(
+                    self._slugify(self.name),
+                    filename
+            ))
+            # If the path makes sense, save the file
+            json_data = json.dumps(data_list, ensure_ascii=False)
+            try:
+                with open(full_filename, mode='wt', encoding='utf-8') as myfile:
+                    myfile.write(json_data)
+            except Exception as e:
+                print(" - Could not save to file `{}`: e=`{}`".format(
+                    full_filename, e
+                ))
+                return False
+
+            return True
+        else:
+            return False
 
     def load_iptv(self):
         """Load XTream IPTV
@@ -515,129 +524,188 @@ class XTream():
           Groups are for all three channel types, Live TV, VOD, and Series
 
         """
+        # If pyxtream has already authenticated the connection and not loaded the data, start loading
+        if (self.state['authenticated'] == True):
+            if (self.state['loaded'] == False):
 
-        for loading_stream_type in (self.live_type, self.vod_type, self.series_type):
-            ## Get GROUPS
+                for loading_stream_type in (self.live_type, self.vod_type, self.series_type):
+                    ## Get GROUPS
 
-            # Try loading local file
-            dt = 0
-            all_cat = self.load_from_file("all_groups_{}.json".format(
-                loading_stream_type
-            ))
-            # If file empty or does not exists, download it from remote
-            if all_cat == None:
-                # Load all Groups and save file locally
-                start = timer()
-                all_cat = self._load_categories_from_provider(loading_stream_type)
-                self.save_to_file(all_cat,"all_groups_{}.json".format(
-                    loading_stream_type
-                ))
-                dt = timer()-start
+                    # Try loading local file
+                    dt = 0
+                    all_cat = self._load_from_file("all_groups_{}.json".format(
+                        loading_stream_type
+                    ))
+                    # If file empty or does not exists, download it from remote
+                    if all_cat == None:
+                        # Load all Groups and save file locally
+                        start = timer()
+                        all_cat = self._load_categories_from_provider(loading_stream_type)
+                        self._save_to_file(all_cat,"all_groups_{}.json".format(
+                            loading_stream_type
+                        ))
+                        dt = timer()-start
 
-            # If we got the GROUPS data, show the statistics and load GROUPS
-            if all_cat != None:
-                print("Loaded {} {} Groups in {:.3f} seconds".format(
-                    len(all_cat),loading_stream_type,dt
-                ))
-                ## Add GROUPS to dictionaries
+                    # If we got the GROUPS data, show the statistics and load GROUPS
+                    if all_cat != None:
+                        print("Loaded {} {} Groups in {:.3f} seconds".format(
+                            len(all_cat),loading_stream_type,dt
+                        ))
+                        ## Add GROUPS to dictionaries
 
-                # Add the catch-all-errors group
-                self.groups.append(self.catch_all_group)
+                        # Add the catch-all-errors group
+                        self.groups.append(self.catch_all_group)
 
-                for cat_obj in all_cat:
-                    # Create Group (Category)
-                    new_group = Group(cat_obj, loading_stream_type)
-                    #  Add to xtream class
-                    self.groups.append(new_group)
-            else:
-                print("Could not load {} Groups".format(loading_stream_type))
-                break
+                        for cat_obj in all_cat:
+                            # Create Group (Category)
+                            new_group = Group(cat_obj, loading_stream_type)
+                            #  Add to xtream class
+                            self.groups.append(new_group)
 
-            ## Get Streams
+                        # Add the catch-all-errors group
+                        self.groups.append(Group({"category_id": "9999","category_name":"xEverythingElse","parent_id":0},loading_stream_type))
 
-            # Try loading local file
-            dt = 0
-            all_streams = self.load_from_file("all_stream_{}.json".format(
-                loading_stream_type
-            ))
-            # If file empty or does not exists, download it from remote
-            if all_streams == None:
-                # Load all Streams and save file locally
-                start = timer()
-                all_streams = self._load_streams_from_provider(loading_stream_type)
-                self.save_to_file(all_streams,"all_stream_{}.json".format(
-                    loading_stream_type
-                ))
-                dt = timer()-start
+                        # Sort Categories
+                        self.groups.sort(key=lambda x: x.name)
+                    else:
+                        print(" - Could not load {} Groups".format(loading_stream_type))
+                        break
 
-            # If we got the STREAMS data, show the statistics and load Streams
-            if all_streams != None:
-                print("Loaded {} {} Streams in {:.3f} seconds".format(
-                    len(all_streams),loading_stream_type,dt
-                ))
-                ## Add Streams to dictionaries
+                    ## Get Streams
 
-                for stream_channel in all_streams:
-                    if stream_channel['name'] != "":
-                        # Generate Group Title
-                        if stream_channel['name'][0].isascii():
+                    # Try loading local file
+                    dt = 0
+                    all_streams = self._load_from_file("all_stream_{}.json".format(
+                        loading_stream_type
+                    ))
+                    # If file empty or does not exists, download it from remote
+                    if all_streams == None:
+                        # Load all Streams and save file locally
+                        start = timer()
+                        all_streams = self._load_streams_from_provider(loading_stream_type)
+                        self._save_to_file(all_streams,"all_stream_{}.json".format(
+                            loading_stream_type
+                        ))
+                        dt = timer()-start
 
-                            # Some channels have no group,
-                            # so let's add them to the catche all group
-                            if stream_channel['category_id'] == None:
-                                stream_channel['category_id'] = '9999'
-                            elif stream_channel['category_id'] != '1':
-                                pass
+                    # If we got the STREAMS data, show the statistics and load Streams
+                    if all_streams != None:
+                        print("Loaded {} {} Streams in {:.3f} seconds".format(
+                            len(all_streams),loading_stream_type,dt
+                        ))
+                        ## Add Streams to dictionaries
 
-                            # Find the first occurence of the group that the
-                            # Channel or Stream is pointing to
-                            the_group = next(
-                                (x for x in self.groups if x.group_id == stream_channel['category_id']),
-                                None
-                            )
+                        skipped_adult_content = 0
+                        skipped_no_name_content = 0
 
-                            if the_group != None:
-                                group_title = the_group.name
-                            else:
-                                group_title = self.catch_all_group.name
-                                the_group = self.catch_all_group
+                        for stream_channel in all_streams:
+                            skip_stream = False
 
-                            if loading_stream_type == self.series_type:
-                                # Load all Series
-                                new_series = Serie(self, stream_channel)
-                                # To get all the Episodes for every Season of each
-                                # Series is very time consuming, we will only
-                                # populate the Series once the user click on the
-                                # Series, the Seasons and Episodes will be loaded
-                                # using x.getSeriesInfoByID() function
+                            # Skip if the name of the stream is empty
+                            if stream_channel['name'] == "":
+                                skip_stream = True
+                                skipped_no_name_content = skipped_no_name_content + 1
+                                self._save_to_file_skipped_streams(stream_channel)
 
-                            else:
-                                new_channel = Channel(
-                                    self,
-                                    group_title,
-                                    stream_channel
+                            # Skip if the user chose to hide adult streams
+                            if self.hide_adult_content and loading_stream_type == self.live_type:
+                                try:
+                                    if stream_channel['is_adult'] == "1":
+                                        skip_stream = True
+                                        skipped_adult_content = skipped_adult_content + 1
+                                        self._save_to_file_skipped_streams(stream_channel)
+                                except:
+                                    print(" - Stream does not have `is_adult` key:\n\t`{}`".format(json.dumps(stream_channel)))
+                                    pass
+
+                            if not skip_stream:
+                                # Some channels have no group,
+                                # so let's add them to the catch all group
+                                if stream_channel['category_id'] == None:
+                                    stream_channel['category_id'] = '9999'
+                                elif stream_channel['category_id'] != '1':
+                                    pass
+
+                                # Find the first occurence of the group that the
+                                # Channel or Stream is pointing to
+                                the_group = next(
+                                    (x for x in self.groups if x.group_id == int(stream_channel['category_id'])),
+                                    None
                                 )
 
-                            # Save the new channel to the local list of channels
-                            if loading_stream_type == self.live_type:
-                                self.channels.append(new_channel)
-                            elif loading_stream_type == self.vod_type:
-                                self.movies.append(new_channel)
-                            else:
-                                self.series.append(new_series)
-
-                            # Add stream to the specific Group
-                            if the_group != None:
-                                if loading_stream_type != self.series_type:
-                                    the_group.channels.append(new_channel)
+                                # Set group title
+                                if the_group != None:
+                                    group_title = the_group.name
                                 else:
-                                    the_group.series.append(new_series)
-                            else:
-                                print("Group not found `{}`".format(stream_channel['name']))
+                                    group_title = self.catch_all_group.name
+                                    the_group = self.catch_all_group
+
+                                if loading_stream_type == self.series_type:
+                                    # Load all Series
+                                    new_series = Serie(self, stream_channel)
+                                    # To get all the Episodes for every Season of each
+                                    # Series is very time consuming, we will only
+                                    # populate the Series once the user click on the
+                                    # Series, the Seasons and Episodes will be loaded
+                                    # using x.getSeriesInfoByID() function
+
+                                else:
+                                    new_channel = Channel(
+                                        self,
+                                        group_title,
+                                        stream_channel
+                                    )
+
+                                if (new_channel.group_id == '9999'):
+                                    print(" - xEverythingElse Channel -> {} - {}".format(new_channel.name,new_channel.stream_type))
+
+                                # Save the new channel to the local list of channels
+                                if loading_stream_type == self.live_type:
+                                    self.channels.append(new_channel)
+                                elif loading_stream_type == self.vod_type:
+                                    self.movies.append(new_channel)
+                                else:
+                                    self.series.append(new_series)
+
+                                # Add stream to the specific Group
+                                if the_group != None:
+                                    if loading_stream_type != self.series_type:
+                                        the_group.channels.append(new_channel)
+                                    else:
+                                        the_group.series.append(new_series)
+                                else:
+                                    print(" - Group not found `{}`".format(stream_channel['name']))
+
+                        # Print information of which streams have been skipped
+                        if self.hide_adult_content:
+                            print(" - Skipped {} adult {} streams".format(skipped_adult_content, loading_stream_type))
+                        if skipped_no_name_content > 0:
+                            print(" - Skipped {} unprintable {} streams".format(skipped_no_name_content, loading_stream_type))
                     else:
-                        print("Stream has empty name `{}`".format(json.dumps(stream_channel)))
+                        print(" - Could not load {} Streams".format(loading_stream_type))
+
+                    self.state['loaded'] = True
+
             else:
-                print("Could not load {} Streams".format(loading_stream_type))
+                print("Warning, data has already been loaded.")
+        else:
+            print("Warning, cannot load steams since authorization failed")
+
+    def _save_to_file_skipped_streams(self, stream_channel: Channel):
+
+        #Build the full path
+        full_filename = osp.join(self.cache_path, "skipped_streams.json")
+
+        # If the path makes sense, save the file
+        json_data = json.dumps(stream_channel, ensure_ascii=False)
+        try:
+            with open(full_filename, mode='a', encoding='utf-8') as myfile:
+                myfile.writelines(json_data)
+        except Exception as e:
+            print(" - Could not save to skipped stream file `{}`: e=`{}`".format(
+                full_filename, e
+            ))
+            return False
 
     def get_series_info_by_id(self, get_series: dict):
         """Get Seasons and Episodes for a Serie
@@ -665,6 +733,35 @@ class XTream():
                         )
                         season.episodes[episode_info['title']] = new_episode_channel
 
+    def _get_request(self, URL: str, timeout: Tuple = (2,15)):
+        """Generic GET Request with Error handling
+
+        Args:
+            URL (str): The URL where to GET content
+            timeout (Tuple, optional): Connection and Downloading Timeout. Defaults to (2,15).
+
+        Returns:
+            [type]: JSON dictionary of the loaded data, or None
+        """
+        try:
+            r = requests.get(URL,timeout=timeout)
+            if r.status_code == 200:
+                return r.json()
+
+        except requests.exceptions.ConnectionError:
+            print(" - Connection Error")
+
+        except requests.exceptions.HTTPError:
+            print(" - HTTP Error")
+
+        except requests.exceptions.TooManyRedirects:
+            print(" - TooManyRedirects")
+
+        except requests.exceptions.ReadTimeout as e:
+            print(" - Timeout while loading data")
+
+        return None
+
     # GET Stream Categories
     def _load_categories_from_provider(self, stream_type: str):
         """Get from provider all category for specific stream type from provider
@@ -685,11 +782,7 @@ class XTream():
         else:
             theURL = ""
 
-        r = requests.get(theURL, timeout=(2,15))
-
-        if r.status_code == 200:
-            return r.json()
-        return None
+        return self._get_request(theURL)
 
     # GET Streams
     def _load_streams_from_provider(self, stream_type: str):
@@ -711,10 +804,7 @@ class XTream():
         else:
             theURL = ""
 
-        r = requests.get(theURL, timeout=(2,15))
-        if r.status_code == 200:
-            return r.json()
-        return None
+        return self._get_request(theURL)
 
     # GET Streams by Category
     def _load_streams_by_category_from_provider(self, stream_type: str, category_id):
@@ -738,10 +828,7 @@ class XTream():
         else:
             theURL = ""
 
-        r = requests.get(theURL)
-        if r.status_code == 200:
-            return r.json()
-        return None
+        return self._get_request(theURL)
 
     # GET SERIES Info
     def _load_series_info_by_id_from_provider(self, series_id: str):
@@ -753,10 +840,8 @@ class XTream():
         Returns:
             [type]: JSON if successfull, otherwise None
         """
-        r = requests.get(self.get_series_info_URL_by_ID(series_id))
-        if r.status_code == 200:
-            return r.json()
-        return r
+        return self._get_request(self.get_series_info_URL_by_ID(series_id))
+
     # The seasons array, might be filled or might be completely empty.
     # If it is not empty, it will contain the cover, overview and the air date
     # of the selected season.
@@ -765,31 +850,24 @@ class XTream():
 
     # GET VOD Info
     def vodInfoByID(self, vod_id):
-        r = requests.get(self.get_VOD_info_URL_by_ID(vod_id))
-        return r
+        return self._get_request(self.get_VOD_info_URL_by_ID(vod_id))
 
     # GET short_epg for LIVE Streams (same as stalker portal,
     # prints the next X EPG that will play soon)
     def liveEpgByStream(self, stream_id):
-        r = requests.get(self.get_live_epg_URL_by_stream(stream_id))
-        if r.status_code == 200:
-            return r.json()
-        return r
+        return self._get_request(self.get_live_epg_URL_by_stream(stream_id))
 
     def liveEpgByStreamAndLimit(self, stream_id, limit):
-        r = requests.get(self.get_live_epg_URL_by_stream_and_limit(stream_id, limit))
-        return r
+        return self._get_request(self.get_live_epg_URL_by_stream_and_limit(stream_id, limit))
 
     #  GET ALL EPG for LIVE Streams (same as stalker portal,
     # but it will print all epg listings regardless of the day)
     def allLiveEpgByStream(self, stream_id):
-        r = requests.get(self.get_all_live_epg_URL_by_stream(stream_id))
-        return r
+        return self._get_request(self.get_all_live_epg_URL_by_stream(stream_id))
 
     # Full EPG List for all Streams
     def allEpg(self):
-        r = requests.get(self.get_all_epg_URL())
-        return r
+        return self._get_request(self.get_all_epg_URL())
 
 
     ## URL-builder methods
